@@ -31,7 +31,7 @@ __author__ = 'Mansour Behabadi'
 __copyright__ = 'Copyright 2011, Mansour Behabadi and Jake Gordon'
 __credits__ = ['Mansour Behabadi', 'Jake Gordon']
 __license__ = 'MIT'
-__version__ = '1.0.14'
+__version__ = '1.0.15'
 __maintainer__ = 'Mansour Behabadi'
 __email__ = 'mansour@oxplot.com'
 
@@ -40,28 +40,57 @@ WILDCARD = '*'
 
 
 class FysomError(Exception):
+
+    '''
+        Raised whenever an unexpected event gets triggered.
+    '''
     pass
 
 
 class Fysom(object):
 
+    '''
+        Wraps the complete finite state machine operations.
+    '''
+
     def __init__(self, cfg):
+        '''
+            Machine construction.
+        '''
         self._apply(cfg)
 
     def isstate(self, state):
+        '''
+            Returns if the given state is the current state.
+        '''
         return self.current == state
 
     def can(self, event):
+        '''
+            Returns if the given event be fired in the current machine state.
+        '''
         return (event in self._map and ((self.current in self._map[event]) or WILDCARD in self._map[event])
                 and not hasattr(self, 'transition'))
 
     def cannot(self, event):
+        '''
+            Returns if the given event cannot be fired in the current state.
+        '''
         return not self.can(event)
 
     def is_finished(self):
+        '''
+            Returns if the state machine is in its final state.
+        '''
         return self._final and (self.current == self._final)
 
     def _apply(self, cfg):
+        '''
+            Does the heavy lifting of machine construction. More notably:
+             >> Sets up the initial and finals states.
+             >> Sets the event methods and callbacks into the same object namespace.
+             >> Prepares the event to state transitions map.
+        '''
         init = cfg['initial'] if 'initial' in cfg else None
         if self._is_base_string(init):
             init = {'state': init}
@@ -74,6 +103,9 @@ class Fysom(object):
         self._map = tmap
 
         def add(e):
+            '''
+                Adds the event into the machine map.
+            '''
             if 'src' in e:
                 src = [e['src']] if self._is_base_string(
                     e['src']) else e['src']
@@ -84,6 +116,8 @@ class Fysom(object):
             for s in src:
                 tmap[e['name']][s] = e['dst']
 
+        # Consider initial state as any other state that can have transition from none to
+        # initial value on occurance of startup / init event ( if specified).
         if init:
             if 'event' not in init:
                 init['event'] = 'startup'
@@ -92,32 +126,43 @@ class Fysom(object):
         for e in events:
             add(e)
 
+        # For all the events as present in machine map, construct the event handler.
         for name in tmap:
             setattr(self, name, self._build_event(name))
 
+        # For all the callbacks, register them into the current object namespace.
         for name in callbacks:
             setattr(self, name, callbacks[name])
 
         self.current = 'none'
 
+        # If initialization need not be deferred, trigger the event for transition to initial state.
         if init and 'defer' not in init:
             getattr(self, init['event'])()
 
     def _build_event(self, event):
-
+        '''
+            For every event in the state machine, prepares the event handler and
+            registers the same into current object namespace.
+        '''
         def fn(*args, **kwargs):
 
             if hasattr(self, 'transition'):
                 raise FysomError(
                     "event %s inappropriate because previous transition did not complete" % event)
+
+            # Check if this event can be triggered in the current state.
             if not self.can(event):
                 raise FysomError(
                     "event %s inappropriate in current state %s" % (event, self.current))
 
+            # On event occurance, source will always be the current state.
             src = self.current
+            # Finds the destination state, after this event is completed.
             dst = ((src in self._map[event] and self._map[event][src]) or
                    WILDCARD in self._map[event] and self._map[event][WILDCARD])
 
+            # Prepares the object will all the meta data to be passed to callbacks.
             class _e_obj(object):
                 pass
             e = _e_obj()
@@ -128,9 +173,12 @@ class Fysom(object):
             setattr(e, 'args', args)
 
             if self.current != dst:
+
+                # Try to trigger the before event, unless it gets canceled.
                 if self._before_event(e) is False:
                     return
 
+                # Wraps the activities that must constitute a single successful transaction.
                 def _tran():
                     delattr(self, 'transition')
                     self.current = dst
@@ -139,39 +187,77 @@ class Fysom(object):
                     self._after_event(e)
                 self.transition = _tran
 
+            # Hook to perform asynchronous transition.
             if self._leave_state(e) is not False:
                 if hasattr(self, 'transition'):
                     self.transition()
 
+        fn.__name__ = event
+        fn.__doc__ = ("Event handler for an {event} event. This event can be " +
+                      "fired if the machine is in {states} states.".format(
+                          event=event, states=self._map[event].keys()))
+
         return fn
 
     def _before_event(self, e):
+        '''
+            Checks to see if the callback is registered before this event can be triggered.
+        '''
         fnname = 'onbefore' + e.event
         if hasattr(self, fnname):
             return getattr(self, fnname)(e)
 
     def _after_event(self, e):
+        '''
+            Checks to see if the callback is registered for, after this event is completed.
+        '''
         for fnname in ['onafter' + e.event, 'on' + e.event]:
             if hasattr(self, fnname):
                 return getattr(self, fnname)(e)
 
     def _leave_state(self, e):
+        '''
+            Checks to see if the machine can leave the current state and perform the transition.
+            This is helpful if the asynchronous job needs to be completed before the machine can
+            leave the current state.
+        '''
         fnname = 'onleave' + e.src
         if hasattr(self, fnname):
             return getattr(self, fnname)(e)
 
     def _enter_state(self, e):
+        '''
+            Executes the callback for onenter_state_ or on_state_.
+        '''
         for fnname in ['onenter' + e.dst, 'on' + e.dst]:
             if hasattr(self, fnname):
                 return getattr(self, fnname)(e)
 
     def _change_state(self, e):
+        '''
+            A general change state callback. This gets triggered at the time of state transition.
+        '''
         fnname = 'onchangestate'
         if hasattr(self, fnname):
             return getattr(self, fnname)(e)
 
     def _is_base_string(self, object):  # pragma: no cover
+        '''
+            Returns if the object is an instance of basestring.
+        '''
         try:
             return isinstance(object, basestring)
         except NameError:
             return isinstance(object, str)
+
+    def trigger(self, event):
+        '''
+            Triggers the given event.
+            The event can be triggered by calling the event handler directly, for ex: fsm.eat()
+            but this method will come in handy if the event is determined dynamically and you have
+            the event name to trigger as a string.
+        '''
+        if not hasattr(self, event):
+            raise FysomError(
+                "There isn't any event registered as %s" % event)
+        return getattr(self, event)()
